#!/usr/bin/lua
require "lunit"
log = require "syslog"
lfs = require "lfs"
json = require "cjson"
local socket = require "socket"
socket.unix = require "socket.unix"
posix = require "posix"

module("processor", package.seeall, lunit.testcase)

function setup()
	-- Setup before each test
	local currentdir = lfs.currentdir()
	package.path = package.path..";"..currentdir.."/share/?.lua;"..currentdir.."/etc/rules/?.lua;"..currentdir.."/etc/rules/custom/?.lua;"..currentdir.."/etc/?.lua;"
	require "config"

	-- Make stdout print to variable "nonstdout"
	-- Note!!! Does not work with print() (print can not be overriden). Specify output as io.stdout and :write to it.
	copy = {
		write = function(self, ...) nonstdout = ... end,
	}
	copy.__index = copy
	io.stdout = setmetatable({mode = "stdout"}, copy)
end

----------------------------------------------
-- Connects to and queries the socket
-- @param cmd string
-- @return string
----------------------------------------------
function connect_and_query_socket(cmd)
	cmd = cmd.."\n"
	local s, sock = pcall(socket.unix)
	if s then
		assert(sock.connect(sock, config.socket_path))
		assert(sock:send(cmd))
		local response = sock:receive()
		return response
	else
		return "die"
	end
end

----------------------------------------------
-- Maybe this test doesn't belong here BUT
-- at least it is not released to production.
----------------------------------------------
function test_no_debug_true_commited()
	assert_false(config.debug, "Do not commit config.debug = true!!!")
end

----------------------------------------------
-- Test for startup errors
-- This also tests that livestatus socket is initiated and config is set.
----------------------------------------------
function test_for_startup_errors()
	-- Hi-jack args
	arg[1] = nil
	local s, processor =  pcall(loadfile, "bin/synergy")
	local output = processor()
	assert_equal("Nothing to do", output, "Expected processor to start up with no errors and return \"Nothing to do\", got "..tostring(output))
end

----------------------------------------------
-- Test that nothing crashes when spool is empty
----------------------------------------------
function test_empty_rule_spool()
	-- Hi-jack args
	arg[1] = "--process_spool"
	-- Set new spool dir
	config.spool_dir = "test/var/"
	local s, processor, m =  pcall(loadfile, "bin/synergy")
	assert(s, "Expected no errors with processor, got errors")
	local status = processor()
	assert_true(status, "Expected function to return true, no such luck I'm afraid")
end

----------------------------------------------
-- Test getting status results
----------------------------------------------
function test_top_and_full()
	-- Run as daemon
	arg[1] = "--monitor"
	-- Don't fork
	arg[3] = "--no-daemon"
	local s, processor = pcall(loadfile, "bin/synergy")
	-- Checks no errors were thrown
	assert_true(s, "Loading of processor failed. Got: "..tostring(processor))

	local pid = posix:fork()
	if pid == -1 then
		fail("Failed to fork in test_top")
	elseif pid == 0 then
		processor()
	else
		-- Give the socket a chance to become available
		socket.select(nil, nil, 2)
		top_response = connect_and_query_socket("top")
		full_response = connect_and_query_socket("full")
		exit = connect_and_query_socket("exit")
	end
	-- Make sure to kill the child
	os.execute("kill "..pid)
	-- Checks that a result was returned that was not false or nil
	assert_string(top_response, "Top failed. Got: "..tostring(response))
	local s, decoded_result = pcall(json.decode, top_response)
	assert_true(s, "Decoding of json failed.")
	assert_table(decoded_result, "No proper json string was returned")
	assert_string(full_response, "Full failed. Got: "..tostring(full_response))
	local s, decoded_result = pcall(json.decode, full_response)
	assert_true(s, "Decoding of json failed.")
	assert_table(decoded_result, "No proper json string was returned")
	assert_equal("Exiting", exit, "Exit command failure")
end

----------------------------------------------
-- Test executing a named rule that doesn't exist
-- Rewrite to utilize socket
----------------------------------------------
function test_execute_named_rule_that_doesnt_exist()
	config.tree_dir = "etc/bps/"
	arg[1] = "nonexistingrule"
	local s, processor = pcall(loadfile, "bin/synergy")
	assert_true(s, "Named execution failed. Got: "..tostring(processor))
	local outcome = processor()
	assert_false(outcome, "There shouldn't have been a rule with supplied name")
	assert(nonstdout, "Function should have returned an error message but didn't")
	local s, decoded_result = pcall(json.decode, nonstdout)
	assert_false(s, "Decoding of json succeded which shouldn't be possible")
	assert_string(decoded_result, "There should be an error string here")
end

----------------------------------------------
-- Test executing a named rule that exists
-- Rewrite to utilize socket
----------------------------------------------
function test_execute_named_rule_that_does_exist()
	config.tree_dir = "etc/bps/"
	arg[1] = "Test-Drive"
	local s, processor = pcall(loadfile, "bin/synergy")
	assert_true(s, "Named execution failed. Got: "..tostring(processor))
	local outcome, full, top = processor()
	assert_true(outcome, "There should have been a rule with supplied name")
	local s, decoded_result = pcall(json.decode, full)
	assert_true(s, "Decoding of json failed")
	assert_table(decoded_result, "No proper json string was returned")
	local s, decoded_result = pcall(json.decode, top)
	assert_true(s, "Decoding of json failed")
	assert_table(decoded_result, "No proper json string was returned")
end

function teardown()
	-- Teardown after each test
	nonstdout = nil
end
